<?php

namespace plugin\Popup\inc;

/** */

class query {

/*

GET NUMBER OF POPUPS

*/

public static function popups( $categories = array() ) {
  return self::have_popups( $categories, array( 'only_count' => '' ) );
}

/*

CHECK IF POPUP EXISTS

*/

public static function popup_exists( $id = '' ) {

global $db;

  if( empty( $id ) ) {
    return false;
  }

  $stmt = $db->stmt_init();
  $stmt->prepare("SELECT COUNT(*) FROM " . DB_TABLE_PREFIX . "popups WHERE id = ?");
  $stmt->bind_param( "i", $id );
  $stmt->execute();
  $stmt->bind_result( $count );
  $stmt->fetch();
  $stmt->close();

  if( $count > 0 ) {
    return true;
  }

  return false;

}

/*

GET INFORMATIONS ABOUT SLIDE

*/

public static function popup_infos( $id = '' ) {

global $db;

  if( empty( $id ) ) {
    return false;
  }

  $stmt = $db->stmt_init();
  $stmt->prepare("SELECT id, type, height, width, title, text, image, href, effect, style, view, live, date FROM " . DB_TABLE_PREFIX . "popups WHERE id = ?");
  $stmt->bind_param( "i", $id );
  $stmt->execute();
  $stmt->bind_result( $id, $type, $height, $width, $title, $text, $image, $href, $effect, $style, $view, $live, $date );
  $stmt->fetch();
  $stmt->close();

  return (object) array( 'ID' => $id, 'type' => $type, 'height' => $height, 'width' => $width, 'title' => $title, 'text' => $text, 'image' => $image, 'href' => $href, 'effect' => $effect, 'style' => $style, 'view' => $view, 'member_view' => ( $view === 1 || $view === 0 ? true : false ), 'visitor_view' => ( $view === 2 || $view === 0 ? true : false ), 'live' => $live, 'date' => $date );

}

/*

NUMBER OF POPUPS

*/

public static function have_popups( $category = array(), $special = array() ) {

global $db;

  $categories = \site\utils::validate_user_data( $category );

  $where = array();

  /*

  WHERE / ORDER BY

  */

  if( isset( $categories['show'] ) ) {
  $show = array_map( 'trim', explode( ',', $categories['show'] ) );
  foreach( $show as $v ) {
    switch( $v ) {
      case 'all': break;
      case 'members': $where[] = 'view IN (0,1)'; break;
      case 'notmembers': $where[] = 'view = 2'; break;
      case 'visitors': $where[] = 'view IN (0,2)'; break;
      case 'notvisitors': $where[] = 'view = 1'; break;
      case 'live': $where[] = 'live > 0'; break;
      case 'notlive': $where[] = 'live = 0'; break;
      default: $where[] = 'live > 0'; break;
    }
  }
  }

  /*

  */

  $stmt = $db->stmt_init();
  $stmt->prepare( "SELECT COUNT(*) FROM " . DB_TABLE_PREFIX . "popups" . ( empty( $where ) ? '' : ' WHERE ' . implode( ' AND ', $where ) ) );
  $stmt->execute();
  $stmt->bind_result( $count );
  $stmt->fetch();
  $stmt->close();

  if( isset( $special['only_count'] ) ) {
    return $count;
  }


  $pags = array();
  $pags['results'] = $count;
  $pags['per_page'] = ( !empty( $categories['per_page'] ) ? (int) $categories['per_page'] : \query\main::get_option( 'items_per_page' ) );
  $pags['pages'] = ceil( $pags['results'] / $pags['per_page'] );
  $page = ( !empty( $_GET['page'] ) ? (int) $_GET['page'] : 1 );
  if( $page < 1 ) $page = 1;
  if( $page > $pags['pages'] ) $page = $_GET['page'] = $pags['pages'];
  $pags['page'] =  $page;
  if( $pags['pages'] > $pags['page'] ) $pags['next_page'] = \site\utils::update_uri( '', array( 'page' => ($pags['page']+1) ) );
  if( $pags['pages'] > 1 && $pags['page'] > 1 ) $pags['prev_page'] = \site\utils::update_uri( '', array( 'page' => ($pags['page']-1) ) );

  return $pags;

}

/*

WHILE THE POPUPS

*/

public static function while_popups( $category = array() ) {

global $db;

  $categories = \site\utils::validate_user_data( $category );

  $where = $orderby = $limit = array();

  if( isset( $categories['max'] ) ) {
    if( !empty( $categories['max'] ) ) {
      $limit[] = $categories['max'];
    }
  } else {
    $page = ( !empty( $_GET['page'] ) ? (int) $_GET['page'] : 1 );
    $per_page = ( isset( $categories['per_page'] ) ? (int) $categories['per_page'] : \query\main::get_option( 'items_per_page' ) );
    $offset = isset( $page ) && $page > 1 ? ( $page - 1 ) * $per_page : 0;

    $limit[] = $offset;
    $limit[] = $per_page;
  }

  /*

  WHERE / ORDER BY

  */

  if( isset( $categories['orderby'] ) ) {
  $order = array_map( 'trim', explode( ',', $categories['orderby'] ) );
  foreach( $order as $v ) {
    switch( $v ) {
      case 'rand': $orderby[] = 'RAND()'; break;
      case 'date': $orderby[] = 'date'; break;
      case 'date desc': $orderby[] = 'date DESC'; break;
    }
  }
  }

  if( isset( $categories['show'] ) ) {
  $show = array_map( 'trim', explode( ',', $categories['show'] ) );
  foreach( $show as $v ) {
    switch( $v ) {
      case 'all': break;
      case 'members': $where[] = 'view IN (0,1)'; break;
      case 'notmembers': $where[] = 'view = 2'; break;
      case 'visitors': $where[] = 'view IN (0,2)'; break;
      case 'notvisitors': $where[] = 'view = 1'; break;
      case 'live': $where[] = 'live > 0'; break;
      case 'notlive': $where[] = 'live = 0'; break;
      default: $where[] = 'live > 0'; break;
    }
  }
  }

  /*

  */

  $stmt = $db->stmt_init();
  $stmt->prepare( "SELECT id, type, height, width, title, text, image, href, effect, style, view, live, date FROM " . DB_TABLE_PREFIX . "popups" . ( empty( $where ) ? '' : ' WHERE ' . implode( ' AND ', $where ) ) . ( empty( $orderby ) ? '' : ' ORDER BY ' . implode( ', ', array_filter( $orderby ) ) ) . ( empty( $limit ) ? '' : ' LIMIT ' . implode( ',', $limit ) ) );
  $stmt->execute();
  $stmt->bind_result( $id, $type, $height, $width, $title, $text, $image, $href, $effect, $style, $view, $live, $date );

  $data = array();
  while( $stmt->fetch() ) {
    $data[] = (object) array( 'ID' => $id, 'type' => $type, 'height' => $height, 'width' => $width, 'title' => $title, 'text' => $text, 'image' => $image, 'href' => $href, 'effect' => $effect, 'style' => $style, 'view' => $view, 'member_view' => ( $view === 1 || $view === 0 ? true : false ), 'visitor_view' => ( $view === 2 || $view === 0 ? true : false ), 'live' => $live, 'date' => $date );
  }

  $stmt->close();

  return $data;

}

}